# -=- encoding: utf-8 -=-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""SQLAlchemy Metadata and Session object"""
from sqlalchemy import MetaData

__all__ = ['engine', 'metadata', 'Session']

# SQLAlchemy database engine.  Updated by model.init_model().
engine = None

# SQLAlchemy session manager.  Updated by model.init_model().
Session = None

# Global metadata. If you have multiple databases with overlapping table 
# names, you'll need a metadata for each database.
metadata = MetaData()

# For use in paster shell:
# from sqlalchemy import create_engine
# model.init_model(create_engine('sqlite:///./vault-database.db', echo=True))



# -*- coding: utf-8 -*-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import transaction

from sflvault.common.crypto import *
from sflvault.lib.vault import SFLvaultAccess, vaultMsg
from sflvault.model import *
import datetime
from decorator import decorator
from datetime import datetime, timedelta
from distutils.version import LooseVersion
import venusian

import sys

log = logging.getLogger(__name__)

MINIMAL_CLIENT_VERSION = LooseVersion('0.7.6')
# Permissions decorators for XML-RPC calls
#

vaultSessions = {}
vault = SFLvaultAccess()

def test_group_admin(request, group_id):
    if not query(Group).filter_by(id=group_id).first():
        return vaultMsg(False, "Group not found: %s" % str(e))
    
    sess = get_session(request['rpc_args'][0], request)

    # Verify if I'm is_admin on that group
    ug = query(UserGroup).filter_by(group_id=group_id,
                                    user_id=sess['user_id']).first()
    me = query(User).get(sess['user_id'])
    
    # Make sure I'm in that group (to be able to decrypt the groupkey)
    if not ug or (not ug.is_admin and not me.is_admin):
        return vaultMsg(False, "You are not admin on that group (nor global admin)")


class XMLRPCDispatcher(object):

    def _dispatch(self, request, method, params):

        params = (request, ) + params

        if method in self.registry:
            return self.registry[method](*params)

    def __init__(self):
        self.registry = {}
        self.scan(sys.modules[__name__])

    def scan(self, module):
        scanner = venusian.Scanner(registry=self.registry)
        scanner.scan(module)


class xmlrpc_method(object):
    """This decorator may be used with pyramid view callables to enable
    them to respond to XML-RPC method calls.

    If ``method`` is not supplied, then the callable name will be used
    for the method name.

    ``_depth`` may be specified when wrapping ``xmlrpc_method`` in another
    decorator. The value should reflect how many stack frames are between
    the wrapped target and ``xmlrpc_method``. Thus a decorator one level deep
    would pass in ``_depth=1``.

    This is the lazy analog to the
    :func:`~pyramid_rpc.xmlrpc.add_xmlrpc_method`` and accepts all of
    the same arguments.

    """
    def __init__(self, method=None, **kw):
        self.method = method
        self.kw = kw

    def __call__(self, wrapped):
        kw = self.kw.copy()
        kw['method'] = self.method or wrapped.__name__

        def callback(scanner, name, ob):
            scanner.registry[kw['method']] = ob


        info = venusian.attach(wrapped, callback, category='pyramid')
        if info.scope == 'class':
            # ensure that attr is set if decorating a class method
            kw.setdefault('attr', wrapped.__name__)

        kw['_info'] = info.codeinfo # fbo action_method
        return wrapped


@decorator
def authenticated_user(func, request, *args, **kwargs):
    """Aborts if user isn't authenticated.

    Timeout check done in get_session.

    WARNING: authenticated_user READ the FIRST non-keyword argument
             (should be authtok)
    """
    cryptok = request['rpc_args'][0]
    ret = _authenticated_user_first(request, cryptok)
    if ret:
        return ret

    return func(request, *args, **kwargs)

def _authenticated_user_first(request, cryptok):
    """DRYed authenticated_user to skip repetition in authenticated_admin"""
    try:
        s = get_session(cryptok, request)
    except SessionNotFoundError:
        s = None
        error_msg = 'session not found'
    except SessionExpiredError:
        s = None
        error_msg = 'session expired'


    if not s:
        return vaultMsg(False, "Permission denied (%s)" % error_msg)

    sess = s

    if 'user_id' in sess:
        vault.myself_id = sess['user_id']
    if 'username' in sess:
        vault.myself_username = sess['username']

@decorator
def authenticated_admin(func, request, *args, **kwargs):
    """Aborts if user isn't admin.

    Check authenticated_user , everything written then applies here as well.
    """
    cryptok = request['rpc_args'][0]
    ret = _authenticated_user_first(request, cryptok)
    if ret:
        return ret
    try:
        sess = get_session(cryptok, request)
    except SessionNotFoundError:
        sess = None

    if sess:
        if not sess['userobj'].is_admin:
            return vaultMsg(False, "Permission denied, admin priv. required")

    return func(request, *args, **kwargs)


@xmlrpc_method(endpoint='sflvault', method='sflvault.authenticate')
def sflvault_authenticate(request, username, cryptok):
    """Receive the *decrypted* cryptok, b64 encoded"""
    settings = request['settings']
    u  = None
    db = None

    # DEPRECATED: will be removed in 0.9
    try:
        if settings['sflvault.vault.session_trust'].lower() in ['1', 'true', 't']:
            # If the session_trust parameter is true trust the session for the authentication.
            try:
                sess = get_session(cryptok, request)
            except SessionNotFoundError:
                sess = None
                print "Session not found... "
            except SessionExpiredError:
                sess = None
                print "Session expired... "

            if sess:
                return vaultMsg(True, 'Authentication successful (cached)', {'authtok': cryptok})
    except KeyError:
        pass
    
    try:
        #u = meta.Session.query(User).filter_by(username=username).one()
        db = meta.Session()
        u = db.query(User).filter(User.username == username).all()[0]
    except:
        return vaultMsg(False, 'Invalid user')

    if u.logging_timeout < datetime.now():
        return vaultMsg(False, 'Login token expired. Now: %s Timeout: %s' % (datetime.now(), u.logging_timeout))

    # str() necessary, to convert buffer to string.
    if cryptok != str(u.logging_token):
        #TODO: Ask about this line.
        #raise Exception
        return vaultMsg(False, 'Authentication failed')
    else:
        newtok = b64encode(randfunc(32))
        set_session(newtok, {'username': username,
                                'timeout': datetime.now() + timedelta(0, int(settings['sflvault.vault.session_timeout'])),
                                'remote_addr': request.get('REMOTE_ADDR', None),
                                'userobj': u,
                                'user_id': u.id
                                })
        return vaultMsg(True, 'Authentication successful', {'authtok': newtok})


@xmlrpc_method(endpoint='sflvault', method='sflvault.login')
def sflvault_login(request, username, version):
    # Require minimal client version.        
    user_version = LooseVersion(version)

    if not version or user_version < MINIMAL_CLIENT_VERSION:
        return vaultMsg(False, "Minimal client version required: '%s'. "\
                        "You announced yourself as version '%s'" % \
                        (MINIMAL_CLIENT_VERSION.vstring, version))

    # Return 'cryptok', encrypted with pubkey.
    # Save decoded version to user's db field.
    #transaction.begin()
    try:
        #u = query(User).filter_by(username=username).one()
        u = meta.Session.query(User).filter_by(username=username).one()
    except Exception, e:
        return vaultMsg(False, "User unknown: %s" % e.message)
    
    # TODO: implement throttling ?

    rnd = randfunc(32)
    # 15 seconds to complete login/authenticate round-trip.
    u.logging_timeout = datetime.now() + timedelta(0, 15)
    u.logging_token = b64encode(rnd)
    if not u.pubkey:
        return vaultMsg(False, "User %s is not set up. Run user-setup first!" % username)
    
    #a = meta.Session.query(User).filter_by(username=username).one()
    e = u.elgamal()
    cryptok = serial_elgamal_msg(e.encrypt(rnd, randfunc(32)))
    
    transaction.commit()
    #meta.Session.close()
    return vaultMsg(True, 'Authenticate please', {'cryptok': cryptok})

@xmlrpc_method(endpoint='sflvault', method='sflvault.user_add')
@authenticated_admin
def user_add(request, authtok, username, is_admin):
    try:
        setup_timeout = request['settings']['sflvault.vault.setup_timeout']
    except KeyError, e:
        setup_timeout = 300
    return vault.user_add(username, is_admin, setup_timeout=setup_timeout)

@xmlrpc_method(endpoint='sflvault', method='sflvault.user_setup')
def user_setup(request, username, pubkey):
    return vault.user_setup(username, pubkey)

@xmlrpc_method(endpoint='sflvault', method='sflvault.user_del')
@authenticated_admin
def sflvault_user_del(request, authtok, user):
    return vault.user_del(user)

@xmlrpc_method(endpoint='sflvault', method='sflvault.user_list')
@authenticated_user
def sflvault_user_list(request, authtok, groups):
    return vault.user_list(groups)

@xmlrpc_method(endpoint='sflvault', method='sflvault.machine_get')
@xmlrpc_method(endpoint='sflvault', method='sflvault.machine.get')
@authenticated_user
def sflvault_machine_get(request, authtok, machine_id):
    return vault.machine_get(machine_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.machine_put')
@authenticated_user
def sflvault_machine_put(request, authtok, machine_id, data):
    return vault.machine_put(machine_id, data)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_get')
@authenticated_user
def sflvault_service_get(request, authtok, service_id, group_id=None):
    return vault.service_get(service_id, group_id)


# si ça arrive via /jsonrpc .. on convertie en JSON en sortant
# si ça arrive via /vault/rpc .. on convertie en XML-RPC
#  XML-RPC faudrait qu'il passe simplement ses paramètres.. SAUF authtok
#  - Donc doit gérer aussi le cas du login, ou le auth-tok est requis.

#@rpc_view(method='sflvault.service_get', skip_first=False)
#def sflvault_service_get(authtok, service_id, group_id=None):

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_get_tree')
@authenticated_user
def sflvault_service_get_tree(request, authtok, service_id, with_groups):
    return vault.service_get_tree(service_id, with_groups)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_put')
@authenticated_user
def sflvault_service_put(request, authtok, service_id, data):
    # 'user_id' required in session.
    # TODO: verify I had access to the service previously.
    sess = get_session(authtok, request)
    req = sql.join(servicegroups_table, usergroups_table,
                    ServiceGroup.group_id==UserGroup.group_id) \
                .join(users_table, User.id==UserGroup.user_id) \
                .select() \
                .where(User.id==sess['user_id']) \
                .where(ServiceGroup.service_id==service_id)
    res = list(meta.Session.execute(req))
    if not res:
        return vaultMsg(False, "You don't have access to that service.")
    else:
        return vault.service_put(service_id, data)

@xmlrpc_method(endpoint='sflvault', method='sflvault.search')
@authenticated_user
def sflvault_search(request, authtok, search_query, group_ids, verbose, filters):
    if group_ids and not filters:
        filters = {'groups': group_ids}
    if group_ids and isinstance(filters, dict) and 'groups' not in filters:
        # Please don't do that, use filters instead.
        filters['groups'] = group_ids

    return vault.search(search_query, filters, verbose)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_add')
@authenticated_user
def sflvault_service_add(request, authtok, machine_id, parent_service_id, url, group_ids, secret,
        notes, metadata):
    return vault.service_add(machine_id, parent_service_id, url, group_ids, secret, notes,
        metadata)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_del')
@authenticated_admin
def sflvault_service_del(request, authtok, service_id):
    return vault.service_del(service_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_list')
@authenticated_user
def sflvault_service_list(request, authtok, machine_id=None, customer_id=None):
    return vault.service_list(machine_id, customer_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.machine_add')
@authenticated_user
def sflvault_machine_add(request, authtok, customer_id, name, fqdn, ip, location, notes):
    return vault.machine_add(customer_id, name, fqdn, ip, location, notes)

@xmlrpc_method(endpoint='sflvault', method='sflvault.machine_del')
@authenticated_admin
def sflvault_machine_del(request, authtok, machine_id):
    return vault.machine_del(machine_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.machine_list')
@authenticated_user
def sflvault_machine_list(request, authtok, customer_id=None):
    return vault.machine_list(customer_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.customer_get')
@xmlrpc_method(endpoint='sflvault', method='sflvault.customer.get')
@authenticated_user
def sflvault_customer_get(request, authtok, customer_id):
    return vault.customer_get(customer_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.customer_put')
@authenticated_user
def sflvault_customer_put(request, authtok, customer_id, data):
    return vault.customer_put(customer_id, data)

@xmlrpc_method(endpoint='sflvault', method='sflvault.customer_add')
@authenticated_user
def sflvault_customer_add(request, authtok, customer_name):
    return vault.customer_add(customer_name)

@xmlrpc_method(endpoint='sflvault', method='sflvault.customer_del')
@authenticated_admin
def sflvault_customer_del(request, authtok, customer_id):
    return vault.customer_del(customer_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.customer_list')
@authenticated_user
def sflvault_customer_list(request, authtok):
    return vault.customer_list()

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_get')
@authenticated_user
def sflvault_group_get(request, authtok, group_id):
    return vault.group_get(group_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_put')
@authenticated_user
def sflvault_group_put(request, authtok, group_id, data):
    return vault.group_put(group_id, data)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_add')
@authenticated_user
def sflvault_group_add(request, authtok, group_name):
    return vault.group_add(group_name)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_del')
@authenticated_admin
def sflvault_group_del(request, authtok, group_id, delete_cascade):
    return vault.group_del(group_id,
                           delete_cascade=delete_cascade)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_add_service')
@authenticated_user
def sflvault_group_add_service(request, authtok, group_id, service_id, symkey):
    return vault.group_add_service(group_id, service_id, symkey)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_del_service')
@authenticated_user
def sflvault_group_del_service(request, authtok, group_id, service_id):
    fail = test_group_admin(request, group_id)
    if fail:
        return fail
    return vault.group_del_service(group_id, service_id)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_add_user')
@authenticated_user
def sflvault_group_add_user(request, authtok, group_id, user, is_admin=False, cryptgroupkey=None):
    return vault.group_add_user(group_id, user, is_admin, cryptgroupkey)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_del_user')
@authenticated_user
def sflvault_group_del_user(request, authtok, group_id, user):
    fail = test_group_admin(request, group_id)
    if fail:
        return fail
    return vault.group_del_user(group_id, user)

@xmlrpc_method(endpoint='sflvault', method='sflvault.group_list')
@xmlrpc_method(endpoint='sflvault', method='sflvault.group.list')
@authenticated_user
def sflvault_group_list(request, authtok, list_users=False):
    return vault.group_list(False, list_users)

@xmlrpc_method(endpoint='sflvault', method='sflvault.service_passwd')
@authenticated_user
def sflvault_service_passwd(request, authtok, service_id, newsecret):
    return vault.service_passwd(service_id, newsecret)

#def _setup_sessions():
#    """DRY out set_session and get_session"""
#    if not hasattr(, 'vaultSessions'):
#        .vaultSessions = {}
def set_session(authtok, value):
    """Sets in 'g.vaultSessions':
    {authtok1: {'username':  , 'timeout': datetime}, authtok2: {}..}
    
    """
 #   _setup_sessions();
    vaultSessions[authtok] = value;

def get_session(authtok, request):
    """Return the values associated with a session"""
    #_setup_sessions();
    
    if not vaultSessions.has_key(authtok):
        raise SessionNotFoundError
        return None

    # XXX: where does the SESSION_TIMEOUT come from?
    if not vaultSessions[authtok].has_key('timeout'):
        vaultSessions[authtok]['timeout'] = datetime.now() + timedelta(0, SESSION_TIMEOUT)

    if vaultSessions[authtok]['timeout'] < datetime.now():
        del(vaultSessions[authtok])
        raise SessionExpiredError
        return None

    if vaultSessions[authtok]['remote_addr'] != request.get('REMOTE_ADDR', 'gibberish'):
        del(vaultSessions[authtok])
        SessionSourceAddressMismatchError
        return None

    return vaultSessions[authtok]

class SessionNotFoundError(Exception):
    pass

class SessionExpiredError(Exception):
    pass

class SessionSourceAddressMismatchError(Exception):
    pass




# -=- encoding: utf-8 -=-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sflvault.tests import TestController

from sflvault.common.crypto import *
from sflvault.common import VaultError

from sflvault.client.client import authenticate
from sflvault.client.client import SFLvaultConfig, SFLvaultClient

import logging
import random

log = logging.getLogger('tester')

class TestVaultController(TestController):
    

    def _rand_name(self):
        return '%s' % str(random.randint(0,1000))

    def _add_new_group(self):
        """Adds a new group with a random name"""
        return self.vault.group_add(self._rand_name())

    def _add_new_customer(self):
        """Adds a new customer with a random name"""
        return self.vault.customer_add(self._rand_name())

    def _add_new_machine(self):
        """Adds a new machine with a random name, linked to a new customer that also have a random name"""
        cid = self._add_new_customer()
        mid = self.vault.machine_add(cid['customer_id'], self._rand_name())
        return mid


    def _add_new_service(self):
        mid = self._add_new_machine()
        gid = self._add_new_group()
        return self.vault.service_add(mid['machine_id'],
                               0,
                               'ssh://sflvault.org',
                               [gid['group_id']],
                               'secret') 
        

    def setUp(self):
        self.vault = self.getVault()

    def test_customer_add(self):
        """testing add a new customer to the vault"""
        res = self.vault.customer_add('testing customer 1 2 3')
        cid1 = res['customer_id']
        self.assertTrue(cid1, 1)
        self.assertEqual(res['message'], 'Customer added')


    def test_machine_add(self):
        """testing add a new machine to the vault"""
        res = self.vault.customer_add(u"Testing é les autres")
        cid2 = res['customer_id']
        self.assertEqual(res['message'], 'Customer added')    
        res = self.vault.machine_add(str(cid2), 
                                     "Machine name 2",
                                     "domain1.example2.com", 
                                     '4.3.2.1',
                                     None, 
                                     None)
        self.assertFalse("Error adding machine" in res['message'])


    def test_service_add(self):
        """testing add a new service to the vault"""
        cres = self.vault.customer_add(u"Testing é les autres")
        mres = self.vault.machine_add(str(cres['customer_id']), 
                                      "Machine name 3",
                                      "domain1.example2.com", 
                                      '4.3.2.1',
                                      None, 
                                      None)
        res = self.vault.service_add(mres['machine_id'],
                                     0, 
                                     'ssh://marrakis@localhost',
                                     [], 
                                     'test',
                                     '')
        self.assertFalse("Error adding service" in res['message'])

    def test_alias_add(self):
        """testing add a new alias to the vault"""
        cres = self.vault.customer_add(u"Testing é les autres")
        ares = self.vault.cfg.alias_add("customer#%s"%cres['customer_id'], "c#%s" % cres['customer_id'])
        self.assertTrue(ares)

    def test_alias_del(self):
        cres = self.vault.cfg.alias_add('customer 1', 'u#1')
        self.assertTrue(cres)
        dres = self.vault.cfg.alias_del('customer 1')
        self.assertTrue(dres)

    def test_user_add(self):
        """testing add a new user to the vault"""
        ures = self.vault.user_add("test_username", False)
        self.assertTrue("User added" in ures['message'])
        ures = self.vault.user_add("test_admin", True)
        self.assertTrue("Admin user added" in ures['message'])

    def test_user_add_extend_expired_setup(self):
        """ If a user is added, and setup_timeout expires, 
        adding it again should extend the setup period"""

        ures = self.vault.user_add("test_username", False)
        self.assertTrue("User added" in ures['message'])
        
        # timeout
        import time; time.sleep(2)

        ures = self.vault.user_add("test_username", False)
        self.assertTrue("had setup timeout expired" in ures['message'])

        
    def test_cannot_readd_user_before_expire(self):
        """ If a user is added, user_add cannot be called agin
        before the setup time expires """

        ures = self.vault.user_add("test_username", False)
        self.assertTrue("User added" in ures['message'])

        self.assertRaises(VaultError,
                          self.vault.user_add,
                          "test_username",
                          False)


    def test_group_add(self):
        """testing add a new group to the vault"""
        gres = self.vault.group_add("test_group")
        self.assertTrue(int(gres['group_id']) > 0)

    def test_group_put_invalid(self):
        self.assertRaises(VaultError,
                          self.vault.group_put,
                          "invalid id",
                          {'name': 'invalid param' })


    def test_group_put_name(self):
        response = self.vault.group_add("other_test_group")
        group_id = response['group_id']
        response2 = self.vault.group_put(group_id, {'name': 'other_test_group_changed'})
        self.assertFalse(response2['error'])

        response3 = self.vault.group_get(group_id)
        self.assertEquals(response3['name'], 'other_test_group_changed')

    def test_group_put_hidden(self):
        response = self.vault.group_add("other_test_group")
        group_id = response['group_id']
        response2 = self.vault.group_put(group_id, {'hidden': True})
        self.assertFalse(response2['error'])

    def test_add_user_and_user_setup(self):
        """testing add a user to the vault and setup passphrase"""
        def givepass():
                return 'passphrase'
        ures1 = self.vault.user_add('testuser', False)
        self.assertTrue(ures1 is not None)
        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)
        ures2 = tmp_vault.user_setup('testuser',
                                     'http://localhost:6555/vault/rpc',
                                     'passphrase')

        self.assertTrue(ures2 is not None)
        

    def test_user_setup_no_user(self):
        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)

        self.assertRaises(VaultError,
                          tmp_vault.user_setup,
                          'invalid user',
                          'http://localhost:6555/vault/rpc',
                          'passphrase')

    def test_user_setup_already_has_public_key(self):
        """ A user cannot do his setup twice """
        ures1 = self.vault.user_add('testuser', False)
        self.assertTrue(ures1 is not None)
        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)
        ures2 = tmp_vault.user_setup('testuser',
                                     'http://localhost:6555/vault/rpc',
                                     'passphrase')

        self.assertTrue(ures2 is not None)

        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)
        self.assertRaises(VaultError,
                          tmp_vault.user_setup,
                          'testuser',
                          'http://localhost:6555/vault/rpc',
                          'passphrase')
   
    def test_user_setup_expired(self):
        import time

        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)

        self.vault.user_add('testuser', False)
        time.sleep(5)

        self.assertRaises(VaultError,
                          tmp_vault.user_setup,
                          'testuser',
                          'http://localhost:6555/vault/rpc',
                          'passphrase')
                                 

    def test_group_add_user(self):
        """testing add a user to a group to the vault"""
        ures1 = self.vault.user_add('test_add_user_group_vault')
        self.assertTrue(ures1 is not None)
        tmp_vault = SFLvaultClient(self.getConfFileUser())
        ures2 = tmp_vault.user_setup('test_add_user_group_vault',
                                     'http://localhost:6555/vault/rpc',
                                     'passphrase')
        self.assertTrue(ures2 is not None)
        
        log.warn(ures1)
        log.warn(ures2)
    
        gres1 = self.vault.group_add("test_group1_user")
        gres2 = self.vault.group_add("test_group2_user")

        gares1 = self.vault.group_add_user(gres1['group_id'], ures1['user_id'])
        gares2 = self.vault.group_add_user(gres2['group_id'], ures1['user_id'], True)

        self.assertTrue("Added user to group successfully" in gares1['message'])
        self.assertFalse("Error adding user to group" in gares2['message'])
    
    def test_group_list(self):
        response = self.vault.group_list()
        self.assertTrue(len(response['list']) == 0)
        self.vault.group_add('test')
        response2 = self.vault.group_list()
        group_list = response2['list']
        self.assertTrue(len(group_list) == 1)
        self.assertTrue(group_list[0]['id'] == 1)
        self.assertTrue(group_list[0]['name'] == 'test')

        self.assertFalse(response['error'])

    def test_group_add_service(self):
        """testing add a service to a group to the vault"""
        gres3 = self.vault.group_add("test_group3_user")
        cres = self.vault.customer_add(u"Testing é les autres")
        mres = self.vault.machine_add(str(cres['customer_id']), 
                                      "Machine name 3",
                                      "domain1.example2.com", 
                                      '4.3.2.1',
                                      None, 
                                      None)
        res = self.vault.service_add(mres['machine_id'],
                                     None, 
                                     'ssh://marrakis@localhost',
                                     [gres3['group_id']], 
                                     'test',
                                     '')
       # gares3 = self.vault.group_add_service(gres3['group_id'], res['service_id'])

        self.assertTrue(res is not None)

    def test_customer_del(self):
        """testing delete a new customer from the vault"""
        cres = self.vault.customer_add(u"Testing é les autres")
        dres = self.vault.customer_del(cres['customer_id'])
        self.assertTrue("Deleted customer c#%s successfully" % cres['customer_id'] in dres['message'])

    def test_service_passwd(self):
        sres = self._add_new_service()
        pres = self.vault.service_passwd(sres['service_id'], 'new_secret')
        self.assertEqual("Password updated for service.", pres['message'])
        self.assertEqual(sres['service_id'], pres['service_id'])

    def test_machine_del(self):
        """testing delete a machine from the vault"""
        cres = self.vault.customer_add(u"Testing é les autres")
        mres = self.vault.machine_add(str(cres['customer_id']), "Machine name 3",
                                      "domain1.example2.com", '4.3.2.1',
                                      None, None)
        dres = self.vault.machine_del(mres['machine_id'])
        self.assertTrue('Deleted machine m#%s successfully' % mres['machine_id'] in dres['message'])

    def test_service_del(self):
        """testing delete a service from the vault"""
        cres = self.vault.customer_add(u"Testing é les autres")
        mres = self.vault.machine_add(str(cres['customer_id']), 
                                      "Machine name 3",
                                      "domain1.example2.com", 
                                      '4.3.2.1',
                                      None, 
                                      None)
        sres = self.vault.service_add(mres['machine_id'],
                                      0, 
                                      u'ssh://service_del',
                                      [], 
                                      'test',
                                      '')
        sres = self.vault.service_del(sres['service_id'])
        self.assertTrue(sres is not None)

        ## Search for service afterwards
        search = self.vault.search('service_del')
        self.assertTrue(len(search['results'].items()) == 0)

    def test_search_invalid_filter_type(self):
        """ A search that uses an invalid filter format should fail
        with a vaultError. However, there is no way to test this 
        behaviour using the client, as it will try to turn any query
        in a dictionary and fail premptively """
        #self.assertRaises(VaultError, 
        #                  self.vault.search,
        #                  'service_del',
        #                  filters=['invalid',])
        pass 

    def test_search_skip_unspecified_and_none_filters(self):
        """ Unspecified filters should be skipped, and they
        should not affect the search"""
        cres = self.vault.customer_add(u"Testing é les autres")
        mres = self.vault.machine_add(str(cres['customer_id']), 
                                      "Machine name 3",
                                      "domain1.example2.com", 
                                      '4.3.2.1',
                                      None, 
                                      None)
        sres = self.vault.service_add(mres['machine_id'],
                                      0, 
                                      u'ssh://service_del',
                                      [], 
                                      'test',
                                      '')
        ## Search for service afterwards
        search = self.vault.search('service_del')
        self.assertTrue(len(search['results'].items()) ==1)

        # Adds all sort of crazy filters!
        search2 = self.vault.search('service_del',
                          filters={
                              # Unspecified filters
                              'color': 'red',
                              'height': '2 meters',
                              'bogomips': 5.5,
                              'anxiety': None,
                              # None values for specified filters
                              'groups': None,
                              'machines': None,
                              'customers': None
                          })

        self.assertTrue(len(search2['results'].items()) ==1)        
                              
    def test_search_fail_on_invalid_filters(self):
        """ When an invalid filter value is specified, it should fail """
        self.assertRaises(VaultError,
                          self.vault.search,
                          'service_del',
                          filters={'machines': ['invalid', 'filter', 'value'] })

    def test_search_filters_narrow_results(self):
        """ Search can be filtered by machine, groups and customers """
        # Adds three services. Each one has a different customer
        # a different machine and a different group.

        service1 = self._add_new_service()
        service2 = self._add_new_service()
        service3 = self._add_new_service()

        self.assertFalse(service1['error'] or
                         service2['error'] or
                         service3['error'])

        # however, they're all for sflvault.org so searching for that
        # should return all three.
        
        search = self.vault.search('sflvault.org')
        self.assertEquals(len(search['results']), 3)

        # Now let's tailor the search to match one of the services
        response = self.vault.service_get(service1['service_id'])
        response2 = self.vault.machine_get(response['machine_id'])

        response3 = self.vault.service_get(service2['service_id'])
        response4 = self.vault.machine_get(response3['machine_id'])

        search2 = self.vault.search('sflvault.org',
                                    filters={
                                        'machines': [response['machine_id']],
                                        'groups': [response['group_id']],
                                        'customers': [response2['customer_id']],
                                    })

        self.assertEquals(len(search2['results']), 1)


        # Multiple search criteria creates the union of the results

        machines = [response['machine_id'], response3['machine_id']]
        groups = [response['group_id'], response3['group_id']]
        customers = [response2['customer_id'], response4['customer_id']]

        search2 = self.vault.search('sflvault.org',
                                    filters={
                                        'machines': machines,
                                        'groups': groups,
                                        'customers': customers
                                    })

        self.assertEquals(len(search2['results']), 2)


    def test_user_del(self):
        """testing delete a user from the vault"""
        cres = self.vault.user_add(u"Utilisateur de test")
        self.assertTrue(cres is not None)
        dres = self.vault.user_del(u"Utilisateur de test")
        self.assertTrue(dres is not None)

    def test_cannot_user_del_twice(self):
        """ Cannot delete a user twice from the vault """
        cres = self.vault.user_add(u"Utilisateur de test")
        self.assertTrue(cres is not None)
        dres = self.vault.user_del(u"Utilisateur de test")
        self.assertRaises(VaultError,
                          self.vault.user_del,
                          u"Utilisateur de test")

    def test_group_del(self):
        """testing delete a group from the vault"""
        cres = self.vault.group_add('test group')
        self.assertTrue(cres is not None)
        dres = self.vault.group_del(cres['group_id'])
        self.assertTrue(dres is not None)

    def test_group_del_cascade(self):
        """ Tests deleting a group and all it's services """

        machine = self._add_new_machine()
        response = self.vault.group_add('test group')
        self.assertFalse(response['error'])

        response2 = self.vault.service_add(machine['machine_id'],
                                          0,
                                          'ssh://sflvault.org',
                                          [response['group_id']],
                                          'secret') 
                                           
        self.assertFalse(response2['error'])

        response4 = self.vault.group_del(response['group_id'],
                                         delete_cascade=True)
        self.assertFalse(response4['error'])

        self.assertRaises(VaultError,
                          self.vault.service_get,
                          response2['service_id'])

    def test_group_del_cascade_error(self):
        response = self.vault.group_add('test group')
        response2 = self._add_new_service()
        self.assertFalse(response['error'] or response2['error'])
        response3 = self.vault.group_add_service(response['group_id'],
                                                 response2['service_id'])

        self.assertRaises(VaultError,
                          self.vault.group_del,
                          response['group_id'])

    def test_group_del_user(self):
        """testing delete a user from a group from the vault"""
        #Add test user
        ures1 = self.vault.user_add('testuser2')
        self.assertTrue(ures1 is not None)
        tmp_vault = SFLvaultClient(self.getConfFileUser(), shell=True)
        ures2 = tmp_vault.user_setup('testuser2',
                                     'http://localhost:6555/vault/rpc',
                                     'passphrase')
        self.assertTrue(ures2 is not None)
        gcres = self.vault.group_add(u'Test group del user')
        uares = self.vault.group_add_user(gcres['group_id'], 'testuser2')
        udres = self.vault.group_del_user(gcres['group_id'], 'testuser2')
        self.assertTrue("Removed user from group successfully" in udres['message'])

    def test_user_list(self):
        response = self.vault.user_list()
        self.assertFalse(response['error'])
        self.assertTrue(len(response['list']) == 1)
        
        admin = response['list'][0]
        self.assertEquals(admin['username'], 'admin')
        self.assertFalse(admin['setup_expired'])
        self.assertFalse(admin['waiting_setup'])
        self.assertTrue('created_stamp' in admin)
        self.assertTrue(admin['is_admin'])
        self.assertEquals(admin['id'], 1)

        # there is initially only one user
        response = self.vault.user_list()
        self.assertFalse(response['error'])
        self.assertEquals(len(response['list']), 1)

        response2 = self.vault.user_add('dcormier')
        self.assertFalse(response2['error'])

        # there should now be two users        
        response3 = self.vault.user_list()
        self.assertFalse(response3['error'])
        self.assertEquals(len(response3['list']), 2)


    def test_service_get_invalid_service(self):
        # Try to get a service that we are sure doesn't
        # exist
        from sflvault.common import VaultError

        self.assertRaises(VaultError,
                          self.vault.service_get,
                          100)

        # Adds a service and retrieves it
        response2 = self._add_new_service()
        response3 = self.vault.service_get(response2['service_id'])
        self.assertEquals(response3['id'], response2['service_id'])
        
        # We shouldn't be able to retrieve this service
        # for a group it is not tied to.

        # FIXME: there is no way to pass group_id in vault.py

        #self.assertRaises(VaultError, self.vault.service_get(response2['service_id'],
        #                                                     group_id='invalid'))

    def test_customer_put_and_get(self):
        response = self._add_new_customer()
        response2 = self.vault.customer_put(response['customer_id'],
                                      {'name': 'dcormier' })
        response3 = self.vault.customer_get(response['customer_id'])

        self.assertEquals(response3['id'],
                          response['customer_id'])

        self.assertEquals(response3['name'],
                          'dcormier')


    def test_machine_put_and_get(self):
        response = self._add_new_machine()
        self.assertFalse(response['error'])
        
        machine_data = { 'ip': '127.0.0.1',
                         'name': 'sflvault',
                         'fqdn': 'sflvault',
                         'location': 'sfl',
                         'notes': 'super machine' }

        response2 = self.vault.machine_put(response['machine_id'],
                                           machine_data)

        machine = self.vault.machine_get(response['machine_id'])
        self.assertEquals(machine['ip'], '127.0.0.1')
        self.assertEquals(machine['name'], 'sflvault')
        self.assertEquals(machine['fqdn'], 'sflvault')
        self.assertEquals(machine['location'], 'sfl')
        self.assertEquals(machine['notes'], 'super machine')
    
    def test_service_put(self):
        response = self._add_new_service()
        my_note = 'new notes'
        my_url = 'http://sflphone.org'
        service_data = {'notes': my_note,
                        'url': my_url }

        response2 = self.vault.service_put(response['service_id'],
                                           service_data)
        self.assertFalse(response2['error'])

        response3 = self.vault.service_get(response['service_id'])

        self.assertEquals(response3['notes'], my_note)
        self.assertEquals(response3['url'], my_url)
        self.assertEquals(response3['id'], response['service_id'])
        
    def test_cannot_put_on_nonexistent_service(self):
        """ Asserts that we cannot call service_put on 
        a nonexistent service """
        self.assertRaises(VaultError,
                          self.vault.service_put,
                          1,
                          {'notes': 'my notes'})
                          

    def test_group_del_service(self):
        group = self._add_new_group()
        service = self._add_new_service()
        
        # there should not be any association between the group and the
        # service
        service_get = self.vault.service_get(service['service_id'],
                                             group_id=group['group_id'])


        self.assertTrue(service_get['group_id'] == '')


        # We add the service to the new group
        group_add_service = self.vault.group_add_service(group['group_id'],
                                                         service['service_id'])
        self.assertFalse(group_add_service['error'])

        # The service should now be part of the group
        service_get2 = self.vault.service_get(service['service_id'])

        

        self.assertTrue(group['group_id'] == service_get2['group_id'])

        # And then we remove it
        service_del = self.vault.group_del_service(group['group_id'],
                                                   service['service_id'])

        # It should not be there anymore
        service_get3 = self.vault.service_get(service['service_id'])
        self.assertTrue(service_get['group_id'] == '')


    def test_group_del_service_from_original_group(self):
        machine = self._add_new_machine()
        original_group = self._add_new_group()

        # if we create a service and add it to an original group
        service = self.vault.service_add(machine['machine_id'],
                                      0,
                                      'ssh://sflvault.org',
                                      [original_group['group_id']],
                                      'secret') 

        # if we try to remove it from this group, there should be an error
        self.assertRaises(VaultError,
                          self.vault.group_del_service,
                          original_group['group_id'],
                          service['service_id'])

        # however, if we add it to another group...
        new_group = self._add_new_group()
        group_add_service = self.vault.group_add_service(new_group['group_id'],
                                                         service['service_id'])
        self.assertFalse(group_add_service['error'])

        # we should be able to remove it from the original group!
        response = self.vault.group_del_service(original_group['group_id'],
                                                service['service_id'])
        self.assertFalse(response['error'])
        

    def test_service_get_tree(self):
        """ tests that we can get a service and it's tree of parent / sub services"""
        response = self._add_new_service()
        response2 = self.vault.service_get_tree(response['service_id'])
        self.assertEquals(len(response2), 1)


    def test_service_get_tree_circular_reference_fail(self):
        """ If there are circular references in service definitions,
        service_get_tree should fail """
        response = self._add_new_service()
        response2 = self._add_new_service()

        service_data = { 'parent_service_id': response2['service_id'] }
        service2_data = { 'parent_service_id': response['service_id'] }

        sput = self.vault.service_put(response['service_id'],
                                      service_data)

        sput2 = self.vault.service_put(response2['service_id'],
                                      service2_data)

        self.assertFalse(sput['error'] or sput2['error'])

        # Now both services should get an error when we try to get them
        self.assertRaises(VaultError,
                          self.vault.service_get_tree,
                          response['service_id'])

        self.assertRaises(VaultError,
                          self.vault.service_get_tree,
                          response2['service_id'])
                          


    def test_customer_list(self):
        # FIXME: remove customer_id from client.py

        customer_add = self.vault.customer_add('dcormier')
        customer_add2 = self.vault.customer_add('dcormier2')
        self.assertFalse(customer_add['error'] or customer_add2['error'])

        customer_list = self.vault.customer_list()['list']
        self.assertEquals(len(customer_list), 2)
        self.assertEquals(customer_list[0]['name'], 'dcormier')
        self.assertEquals(customer_list[1]['name'], 'dcormier2')
    def test_machine_list(self):
        machine1 = self._add_new_machine()
        machine2 = self._add_new_machine()
        
        machine_list_response = self.vault.machine_list()
        self.assertFalse(machine_list_response['error'])
        
        machine_list = machine_list_response['list']
        self.assertEquals(len(machine_list), 2)


    def test_machine_list_of_customer(self):

        # adds a customer and a machine
        customer = self.vault.customer_add('dcormier')
        machine1 = self._add_new_machine()

        # customer should not have any machine
        machine_list_response = self.vault.machine_list(customer_id=customer['customer_id'])
        machine_list = machine_list_response['list']
        self.assertEquals(len(machine_list), 0)

        # changes the owner of the machine to the previously added customer
        machine_put = self.vault.machine_put(machine1['machine_id'],
                                             {'customer_id': customer['customer_id']})

        self.assertFalse(machine_put['error'])
        
        # customer should now have one machine
        machine_list_response2 = self.vault.machine_list(customer_id=customer['customer_id'])
        machine_list2 = machine_list_response2['list']
        self.assertEquals(len(machine_list2), 1)
        self.assertEquals(machine_list2[0]['customer_id'],
                          customer['customer_id'])

        # let's change the owner again to an invalid customer
        machine_put2 = self.vault.machine_put(machine1['machine_id'],
                                              {'customer_id': 37})

        self.assertFalse(machine_put2['error'])

        # our previous customer should not have any machine
        machine_list_response3 = self.vault.machine_list(
            customer_id=customer['customer_id'])

        machine_list3 = machine_list_response3['list']
        self.assertEquals(len(machine_list3), 0)




# -=- encoding: utf-8 -=-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from sqlalchemy import types

class JSONEncodedDict(types.TypeDecorator):
    """Represents an mutable structure as a json-encoded string.

    Usage::

        JSONEncodedDict(255)

    """

    impl = types.Text

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        return json.loads(value) if value else {}

    def copy_value(self, value):
        return json.loads(simplejson.dumps(value))



import ConfigParser
from datetime import datetime, timedelta
from SimpleXMLRPCServer import SimpleXMLRPCServer, SimpleXMLRPCRequestHandler, SimpleXMLRPCDispatcher
import SocketServer
from BaseHTTPServer import HTTPServer
import os
import sys
import logging
import logging.config
import argparse
import socket

import transaction
from sqlalchemy import engine_from_config
from OpenSSL import SSL

import sflvault.model
from sflvault.views import XMLRPCDispatcher

log = logging.getLogger(__name__)

# SSL handling based on http://code.activestate.com/recipes/496786-simple-xml-rpc-server-over-https/

class SFLvaultRequestHandler(SimpleXMLRPCRequestHandler):

    def __init__(self, request, client_address, server):
        self.client_address = client_address
        SimpleXMLRPCRequestHandler.__init__(self, request, client_address, server)

    def setup(self):
        self.connection = self.request
        self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
        self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)

    def _dispatch(self, method, params):
        address = self.client_address

        request = {
            'REMOTE_ADDR': address[0],
            'PORT': address[1],
            'rpc_args': params,
            'settings': SFLvaultServer.settings,
        }

        return self.server.instance._dispatch(request, method, params)

    rpc_paths = ('/vault', '/vault/rpc', '/',)

class SecureXMLRPCServer(HTTPServer, SimpleXMLRPCDispatcher):
    def __init__(self, server_address, requestHandler, keyfile, certfile, logRequests=True, allow_none=False):
        """Secure XML-RPC server.

        It it very similar to SimpleXMLRPCServer but it uses HTTPS for transporting XML data.
        """
        self.logRequests = logRequests

        SimpleXMLRPCDispatcher.__init__(self, allow_none=allow_none)
        SocketServer.BaseServer.__init__(self, server_address, requestHandler)
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.use_privatekey_file(keyfile)
        ctx.use_certificate_file(certfile)
        self.socket = SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
        self.server_bind()
        self.server_activate()

    def shutdown_request(self, request):
        # Our SSL connection doesn't take an argument on shutdown so we have to override the method
        # that calls it.
        try:
            #explicitly shutdown.  socket.close() merely releases
            #the socket and waits for GC to perform the actual close.
            request.shutdown()
        except socket.error:
            pass #some platforms may raise ENOTCONN here
        self.close_request(request)

class SFLvaultServer(object):

    def __init__(self, config_file_name):
        self.server = None
        SFLvaultServer.settings = self.get_settings(config_file_name)

        self.start_sqlalchemy()
        self.initialize_models()
        self.create_admin_if_necessary()
        self.initialize_server()
        
    def get_settings(self, config_file_name=None):
        result = {
            'sflvault.vault.session_timeout': '15',
            'sflvault.vault.setup_timeout': '300',
            'sflvault.host': 'localhost',
            'sflvault.port': '5000',
            'sqlalchemy.url': 'sqlite:///%s/sflvault.db' % os.getcwd()
        }
        if config_file_name:
            config = ConfigParser.ConfigParser()
            config.read(config_file_name)
            result.update(self.get_dict_for_config_section(config, 'sflvault'))
        return result

    def start_sqlalchemy(self):
        self.engine = engine_from_config(SFLvaultServer.settings,
                                    'sqlalchemy.')

    def initialize_models(self):
        sflvault.model.init_model(self.engine)
        sflvault.model.meta.metadata.create_all(self.engine)

    def create_admin_if_necessary(self):
        if not sflvault.model.query(sflvault.model.User).filter_by(username='admin').first():
            log.info ("It seems like you are using SFLvault for the first time. An\
                'admin' user account will be added to the system.")
            u = sflvault.model.User()
            u.waiting_setup = datetime.now() + timedelta(0,900)
            u.username = u'admin'
            u.created_time = datetime.now()
            u.is_admin = True
            #transaction.begin()
            sflvault.model.meta.Session.add(u)
            transaction.commit()

    def initialize_server(self):
        dispatcher = self._create_request_dispatcher()
        host = SFLvaultServer.settings['sflvault.host']
        port = int(SFLvaultServer.settings['sflvault.port'])
        address = (host, port)
        keyfile = SFLvaultServer.settings.get('sflvault.keyfile')
        certfile = SFLvaultServer.settings.get('sflvault.certfile')
        if keyfile and certfile:
            log.info("Starting in SSL mode")
            self.server = SecureXMLRPCServer(
                address,
                requestHandler=SFLvaultRequestHandler,
                keyfile=keyfile,
                certfile=certfile,
                logRequests=False,
                allow_none=True,
            )
        else:
            log.info("Starting in insecure mode")
            self.server = SimpleXMLRPCServer(
                address,
                requestHandler=SFLvaultRequestHandler,
                logRequests=False,
                allow_none=True,
            )
        self.server.register_introspection_functions()
        self.server.register_instance(dispatcher)

    def _create_request_dispatcher(self):
        dispatcher = XMLRPCDispatcher()
        dispatcher.scan(sflvault.views)
        return dispatcher

    def start_server(self):
        self.server.serve_forever()

    def get_dict_for_config_section(self, config, section):
        my_dict = {} 
        for key in config._sections[section].keys():
            my_dict[key] = config.get(section, key, 0,
                                      {'here': os.getcwd()})
        return my_dict

def main():
    parser = argparse.ArgumentParser(description="Launch the SFLVault server")
    parser.add_argument('config_file', nargs='?', default=None,
        help="INI config file")
    args = parser.parse_args()
    if args.config_file:
        logging.config.fileConfig(args.config_file)
    server = SFLvaultServer(args.config_file)
    server.start_server()

if __name__ == '__main__':
    main()



# -=- encoding: utf-8 -=-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Pylons application test package

When the test runner finds and executes tests within this directory,
this file will be loaded to setup the test environment.

It registers the root directory of the project in sys.path and
pkg_resources, in case the project hasn't been installed with
setuptools. It also initializes the application via websetup (paster
setup-app) with the project's test.ini configuration file.
"""

# WARNING: For the moment, running the tests only works if you installed all sflvault packages
# (common, client, server) in "developer mode". If you don't, you'll have problems with namespace
# packaging and imports won't work.

import os
import sys
from unittest import TestCase


import threading

from sflvault.client import SFLvaultClient
from sflvault.lib.vault import SFLvaultAccess
from sflvault.server import SFLvaultServer
import logging
log = logging.getLogger(__name__)
__all__ = ['url_for', 'TestController', 'setUp', 'tearDown']

here_dir = os.path.dirname(os.path.abspath(__file__))
conf_dir = os.path.dirname(os.path.dirname(here_dir))

sys.path.insert(0, conf_dir)


dbfile = os.path.join(conf_dir, 'test-database.db')
confile = os.path.join(conf_dir, 'test-config')
userconfile = os.path.join(conf_dir, 'test-config-user')
test_file = os.path.join(conf_dir, 'test.ini')
globs = {}

def tearDown():
    """Close the SFLVault server"""
    #globs['server'].server_close()
    pass

def getConfFileAdmin():
    if (os.path.exists(confile)):
        os.unlink(confile)
    return confile 


def setUp():
    """Setup the temporary SFLvault server"""
    # Remove the test database on each run.
    if os.path.exists(dbfile):
        os.unlink(dbfile)
    # Remove the test config on each run
    if os.path.exists(confile):
        os.unlink(confile)

    os.environ['SFLVAULT_IN_TEST'] = 'true'

 
    server = SFLvaultServer(test_file)

    t = threading.Thread(target=server.start_server)
    t.setDaemon(True)
    t.start()

def delete_all_groups(self):

    vault_response = self.group_list()
    groups = vault_response['list']
    for group in groups:
        print "*** Deleting group: %s" % group['id']
        delete_cascade = True
        self.group_del(group['id'], delete_cascade=True)

def delete_all_users(self):

    vault_response = self.user_list()
    for user in vault_response['list']:
        if not (user['username'] == 'admin'):
            print "*** Deleting user: %s" % user['id']
            self.user_del(user['username'])

def delete_all_machines(self):
    vault_response = self.machine_list()
    for machine in vault_response['list']:
        print "*** Deleting machine: %s" % machine['id']
        self.machine_del(machine['id'])

def delete_all_customers(self):
    vault_response = self.customer_list()
    for customer in vault_response['list']:
        print "*** Deleting customer: %s" % customer['id']
        self.customer_del(customer['id'])


class TestController(TestCase):

    def __init__(self, *args, **kwargs):
        TestCase.__init__(self, *args, **kwargs)
        
    def getConfFileUser(self):
        if os.path.exists(userconfile):
            os.unlink(userconfile)
        return userconfile

    def tearDown(self):
        globs['vault'].delete_all_groups()
        globs['vault'].delete_all_users()
        globs['vault'].delete_all_machines()
        globs['vault'].delete_all_customers()

    def getVault(self):
        """Get the SFLVault server vault"""
        if 'vault' not in globs:
            SFLvaultClient.delete_all_groups = delete_all_groups
            SFLvaultClient.delete_all_users = delete_all_users
            SFLvaultClient.delete_all_machines = delete_all_machines
            SFLvaultClient.delete_all_customers = delete_all_customers

            vault =  SFLvaultClient(getConfFileAdmin(), shell=True)

            globs['vault'] = vault
            passphrase = u'test'
            username = u'admin'    

            def givepass():
                return passphrase        
            globs['vault'].set_getpassfunc(givepass)
            log.warn("testing user setup")
            globs['vault'].user_setup(username,
                'http://localhost:6555/vault/rpc', passphrase)
            globs['cfg'] = globs['vault'].cfg
        return globs['vault']

    



# -=- encoding: utf-8 -=-
#
# SFLvault - Secure networked password store and credentials manager.
#
# Copyright (C) 2008-2009  Savoir-faire Linux inc.
#
# Author: Alexandre Bourget <alexandre.bourget@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'SQLAlchemy',
    'venusian',
    'zope.sqlalchemy',
    'decorator',
    'pyOpenSSL',
]

setup(
    name='SFLvault-server',
    version='0.8.0.0',
    description='Networked credentials store and authentification manager - Server',
    author='Alexandre Bourget',
    author_email='alexandre.bourget@savoirfairelinux.com',
    url='http://www.sflvault.org',
    long_description=README + '\n\n' +  CHANGES,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
    ],
    packages=find_packages(),
    namespace_packages=['sflvault'],
    include_package_data=True,
    zip_safe=False,
    package_data={'sflvault': ['i18n/*/LC_MESSAGES/*.mo']},
    test_suite='sflvault',
    install_requires = requires,
)

